@@ -6,6 +6,10 @@ from . import DataType
 from .objects import Variable, Record
 
 
+class EDSError(Exception):
+    '''Error with EDS'''
+
+
 @dataclass
 class FileInfo:
 
@@ -19,6 +23,18 @@ class FileInfo:
     modification_dt: datetime = datetime.now()
     modified_by: str = ''
 
+    def __eq__(self, other) -> bool:
+        # ignore file_name, modification_dt, and modified_by
+
+        if self.file_version != other.file_version \
+                or self.file_revision != other.file_revision \
+                or self.description != other.description \
+                or self.creation_dt != other.creation_dt \
+                or self.created_by != other.created_by:
+            return False
+
+        return True
+
 
 @dataclass
 class DeviceInfo:
@@ -48,6 +64,25 @@ class DeviceInfo:
     num_of_tpdo: int = 0
     lss_supported: bool = False
 
+    def __eq__(self, other) -> bool:
+        if self.vendor_name != other.vendor_name \
+                or self.vendor_number != other.vendor_number \
+                or self.product_name != other.product_name \
+                or self.product_number != other.product_number \
+                or self.order_code != other.order_code \
+                or self.baud_rate != other.baud_rate \
+                or self.simple_boot_up_master != other.simple_boot_up_master \
+                or self.simple_boot_up_slave != other.simple_boot_up_slave \
+                or self.grandularity != other.grandularity \
+                or self.dynamic_channel_supperted != other.dynamic_channel_supperted \
+                or self.group_messaging != other.group_messaging \
+                or self.num_of_rpdo != other.num_of_rpdo \
+                or self.num_of_tpdo != other.num_of_tpdo \
+                or self.lss_supported != other.lss_supported:
+            return False
+
+        return True
+
 
 @dataclass
 class DeviceCommissioning:
@@ -60,6 +95,18 @@ class DeviceCommissioning:
     canopen_manager: bool = False
     lss_serialnumber: int = 0
 
+    def __eq__(self, other) -> bool:
+        if self.node_id != other.node_id \
+                or self.node_name != other.node_name \
+                or self.baud_rate != other.baud_rate \
+                or self.net_number != other.net_number \
+                or self.network_name != other.network_name \
+                or self.canopen_manager != other.canopen_manager \
+                or self.lss_serialnumber != other.lss_serialnumber:
+            return False
+
+        return True
+
 
 class EDS:
 
@@ -87,6 +134,21 @@ class EDS:
         self.canopennode = False  # flag for canopennode eds/dcf
         self._storage_locations = []
 
+    def __eq__(self, other) -> bool:
+        if self.indexes != other.indexes \
+                or self.file_info != other.file_info \
+                or self.device_info != other.device_info \
+                or self.device_commissioning != other.device_commissioning \
+                or self.comment != other.comment \
+                or self.canopennode != other.canopennode:
+            return False
+
+        for i in self.indexes:
+            if self[i] != other[i]:
+                return False
+
+        return True
+
     def __len__(self):
         return len(self._data)
 
@@ -95,7 +157,10 @@ class EDS:
 
     def __setitem__(self, index: int, item):
         if index in self._data:
-            raise ValueError(f'index 0x{index:X} already exist')
+            raise EDSError(f'index 0x{index:X} already exist')
+
+        if item.storage_location == '' and self._storage_locations:
+            item.storage_location = self._storage_locations[0]
 
         self._data[index] = item
 
@@ -105,14 +170,19 @@ class EDS:
     def insert(self, index: int, subindex: int, item):
         '''Insert a object into the object dictionary'''
 
-        if subindex is None:  # use only index
+        if subindex is None:
             if index in self._data:
-                raise ValueError(f'index 0x{index:X} already exist')
+                raise EDSError(f'index 0x{index:X} already exist')
             self._data[index] = item
-        elif subindex in self._data:  # use index and subindex
-            raise ValueError(f'subindex 0x{subindex:X} already exist for index 0x{index:X}')
+        elif index not in self._data:
+            raise EDSError(f'cannot insert subindex 0x{subindex:X} for index 0x{index:X}, as that '
+                           'index does not exist')
+        elif isinstance(self._data[index], Variable):
+            raise EDSError('cannot insert a subindex into a Variable')
+        elif subindex in self._data[index].subindexes:
+            raise EDSError(f'subindex 0x{subindex:X} already exist for index 0x{index:X}')
         elif not isinstance(item, Variable):
-            raise ValueError('cannot insert non-Variable into subindex')
+            raise EDSError('cannot insert non-Variable into subindex')
         else:
             self._data[index][subindex] = item
 
@@ -135,7 +205,7 @@ class EDS:
                 break
 
         if next_rpdo == -1:
-            raise ValueError('No more RPDO slots')
+            raise EDSError('No more RPDO slots')
 
         temp = next_rpdo % 4
         if temp == 0:
@@ -174,7 +244,7 @@ class EDS:
                 break
 
         if next_tpdo == -1:
-            raise ValueError('No more TPDO slots')
+            raise EDSError('No more TPDO slots')
 
         temp = next_tpdo % 4
         if temp == 0:
@@ -275,5 +345,39 @@ class EDS:
 
     def add_storage_location(self, storage_location: str):
 
-        if storage_location not in self._storage_locations:
+        if storage_location not in self._storage_locations and storage_location != '':
             self._storage_locations.append(storage_location)
+
+    def copy_object(self, index: int, subindex: int, new_index: int, new_subindex: int,
+                    move: bool = False):
+        '''Move or copy an object in the OD'''
+
+        if index not in self.indexes:
+            raise EDSError(f'no object exists at index 0x{index:X}')
+        if subindex is not None and subindex not in self._data[index].subindexes:
+            raise EDSError(f'no object exists at index 0x{index:X} subindex 0x{subindex:02X}')
+
+        if new_subindex is None and new_index in self._data:
+            raise EDSError(f'object already exist at index 0x{new_index:X}')
+        if new_subindex is not None:
+            if isinstance(self._data[new_index], Variable):
+                raise EDSError('cannot move an object to a subindex of an Variable')
+            if new_subindex in self._data[new_index].subindexes:
+                raise EDSError(f'object already exist at index 0x{new_index:X} subindex '
+                               f'0x{new_subindex:02X}')
+
+        if not isinstance(self._data[index], Variable) and new_subindex is not None:
+            raise EDSError('cannot move a non-Variable to a subindex')
+
+        obj = self._data[index] if subindex is None else self._data[index][subindex]
+
+        if new_subindex is None:
+            self._data[new_index] = obj
+        else:
+            self._data[new_index][new_subindex] = obj
+
+        if move:
+            if subindex is None:
+                del self._data[index]
+            else:
+                del self._data[index][subindex]
