@@ -15,7 +15,7 @@ class Variable:
     data_type: DataType = DataType.UNSIGNED32
     low_limit: str = ''
     high_limit: str = ''
-    default_value: str = ''
+    default_value: str = '0'
     access_type: AccessType = AccessType.RW
     pdo_mapping: bool = False
     object_type: ObjectType = ObjectType.VAR
@@ -50,6 +50,27 @@ class Record:
             )
         }
 
+    def __eq__(self, other) -> bool:
+
+        if self.parameter_name != other.parameter_name:
+            return False
+        if self.denotation != other.denotation:
+            return False
+        if self.comments != other.comments:
+            return False
+        if self.object_type != other.object_type:
+            return False
+        if self.storage_location != other.storage_location:
+            return False
+        if self.subindexes != other.subindexes:
+            return False
+
+        for subindex in self.subindexes:
+            if self[subindex] != other[subindex]:
+                return False
+
+        return True
+
     def __len__(self) -> int:
         return len(self._data)
 
@@ -65,6 +86,7 @@ class Record:
         elif subindex in self._data:  # add subindex
             raise ValueError('Subindex already exists')
         else:
+            variable.storage_location = self.storage_location
             self._data[subindex] = variable
 
             # update record size subindex
@@ -121,6 +143,16 @@ class Array(Record):
         self.object_type = ObjectType.ARRAY
         self._data_type = data_type
 
+    def __eq__(self, other) -> bool:
+
+        if not super().__eq__(other):
+            return False
+
+        if self.data_type != other.data_type:
+            return False
+
+        return True
+
     def __setitem__(self, subindex: int, variable: Variable):
         '''Add a subindex to the array'''
 
@@ -132,6 +164,7 @@ class Array(Record):
         elif self._data_type and variable.data_type != self._data_type:
             raise ValueError('Variable\'s data type does not match array\'s data type')
         else:
+            variable.storage_location = self.storage_location
             self._data[subindex] = variable
 
             # set the data_type if not set
